'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpackNodeExternals = require('webpack-node-externals');

var _webpackNodeExternals2 = _interopRequireDefault(_webpackNodeExternals);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackSystemRegister = require('webpack-system-register');

var _webpackSystemRegister2 = _interopRequireDefault(_webpackSystemRegister);

var _uglifyjsWebpackPlugin = require('uglifyjs-webpack-plugin');

var _uglifyjsWebpackPlugin2 = _interopRequireDefault(_uglifyjsWebpackPlugin);

var _defaults = require('lodash/defaults');

var _defaults2 = _interopRequireDefault(_defaults);

var _webpackVisualizerPlugin = require('webpack-visualizer-plugin');

var _webpackVisualizerPlugin2 = _interopRequireDefault(_webpackVisualizerPlugin);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * webpack client config
 * @param configFile
 * @param target
 */
var createWebpackConfig = function createWebpackConfig(configFile) {
  /**
   * @deprecated devUnpkgOrigin and unpkgOrigin will be deprecated
   */
  if (!configFile.publicPathPrefix && configFile.unpkgOrigin) {
    console.warn('\'unpkgOrigin\' will be deprecated');
    configFile.publicPathPrefix = configFile.unpkgOrigin;
  }

  (0, _defaults2.default)(configFile, {
    __DEV__: process.env.NODE_ENV !== 'production',
    context: process.cwd(),
    platform: 'web',
    entry: './src/index.js',
    nodeModulesDir: './node_modules',
    packageFile: './package.json',
    outputDir: './umd/',
    publicPathPrefix: 'https://cdn.jsdelivr.net/npm'
  });

  var __DEV__ = configFile.__DEV__,
      context = configFile.context,
      platform = configFile.platform,
      nodeModulesDir = configFile.nodeModulesDir,
      packageFile = configFile.packageFile,
      outputDir = configFile.outputDir,
      publicPathPrefix = configFile.publicPathPrefix;


  var packageJSON = JSON.parse(_fs2.default.readFileSync(_path2.default.resolve(context, packageFile), 'utf8'));

  var nodeModulesPath = _path2.default.resolve(context, nodeModulesDir) + _path2.default.sep;
  var outputPath = _path2.default.resolve(context, outputDir) + _path2.default.sep;
  var publicPath = _url2.default.resolve(__DEV__ ? '/' : publicPathPrefix, '') + _path2.default.posix.resolve('/' + packageJSON.name + '@' + (__DEV__ ? 'latest' : packageJSON.version), outputDir).substr(1) + _path2.default.sep;
  var entryName = _path2.default.basename(packageJSON.name);

  var config = {
    context: context,
    devtool: false,
    node: {
      fs: 'empty'
    },
    entry: _defineProperty({}, entryName, [_path2.default.resolve(context, configFile.entry)]),
    target: platform,
    output: {
      path: outputPath,
      publicPath: publicPath, // css中的图片地址的前缀, 可以加上域名
      filename: '[name].' + (__DEV__ ? 'development' : 'production') + '.js',
      library: packageJSON.name,
      libraryTarget: platform === 'web' ? 'umd' : 'commonjs2',
      umdNamedDefine: platform === 'web'
    },
    externals: platform === 'node' ? [(0, _webpackNodeExternals2.default)()] : {},
    resolve: {
      alias: {},
      extensions: ['.jsx', '.js', '.json'],
      modules: ['node_modules', _path2.default.resolve(context, 'node_modules')]
    },
    module: {
      rules: [{
        test: /\.(png|jpg|jpeg|svg|gif|bmp|jpeg)$/,
        loader: 'url-loader?limit=1024&name=images/[hash].[ext]'
      }, {
        test: /\.(json)$/,
        loader: 'json-loader'
      }, {
        test: /\.hash\.css$/,
        use: ['style-loader', 'css-loader?modules&importLoaders=1&localIdentName=[name]__[local]___[hash:base64:5]']
      }, {
        test: /^((?!hash).)*\.css$/,
        use: ['style-loader', 'css-loader']
      }, {
        test: /(\.js|\.jsx)$/,
        exclude: /node_modules/,
        loader: 'babel-loader'
      }]
    },
    plugins: []
  };

  if (configFile.useSystemRegister) {
    // console.log('lolla: use system register')
    config.plugins.push(new _webpackSystemRegister2.default({}));
  }

  if (configFile.externals) {
    configFile.externals.forEach(function (externalItem) {
      config.externals[externalItem.commonjs] = Object.assign({
        root: externalItem.commonjs
      }, externalItem);
    });
    // console.log(config.externals)
  }

  if (configFile.alias) {
    configFile.alias.forEach(function (aliasItem) {
      config.resolve.alias[aliasItem.commonjs] = _path2.default.resolve(nodeModulesPath, aliasItem.path);
      // console.log(config.resolve.alias[aliasItem])
    });
  }

  if (configFile.compress) {
    // todo use uglify before bug (https://github.com/babel/babili/issues/583) fixed
    config.plugins.push(new _uglifyjsWebpackPlugin2.default());
    // config.plugins.push(new BabiliPlugin({
    //   removeConsole: true,
    //   keepFnName: true,
    //   keepClassName: true,
    // }, {
    //   test: /\.js($|\?)/i
    // }))
  }
  if (__DEV__) {
    // console.log(`[webpack configure] use define plugin, NODE_ENV: development`)
    config.plugins.push(new _webpack2.default.DefinePlugin({
      'process.env.NODE_ENV': JSON.stringify('development')
    }));
  } else {
    // console.log(`[webpack configure] use define plugin, NODE_ENV: production`)
    // config.plugins.push(
    //   new StatsPlugin(`./stats.json`, {
    //     chunkModules: true,
    //     exclude: configFile.externals.map(name => {
    //       return new RegExp(`/node_modules[\\\/]${name}/`)
    //     })
    //   })
    // )
    config.plugins.push(new _webpackVisualizerPlugin2.default({
      filename: './stats.html'
    }));
    config.plugins.push(new _webpack2.default.DefinePlugin({
      'process.env.NODE_ENV': JSON.stringify('production')
    }));
  }

  return config;
};

exports.default = createWebpackConfig;